/*
 * Copyright (c) 2008, Noel Lemouel
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of this project nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *    
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND THE CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR THE CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* TODO clean this code ;-) */

#include <font_8x8.c>
#include <fbcon.h>

#include <sysutil.h>

/* 8x8 font */
#define font_w 8
#define font_h 8

#define TERM_FT_COLOR 0x80ffffff
#define TERM_BG_COLOR 0x80000000

struct term {
   int x_max_chr;
   int y_max_chr;
   int cursor_x;
   int cursor_y;
   int cursor_visible;
   int cursor_locked;
};

struct fb {
   int width;
   int height;
   unsigned int* base;
};

static struct fb mmapfb;
volatile static struct term fbterm; /* needed inside an irq routine */

void toggle_cursor(void)
{
   if(fbterm.cursor_locked) return;

   if(fbterm.cursor_visible)
   {
      charblt(0x00, fbterm.cursor_x, fbterm.cursor_y);
      fbterm.cursor_visible = 0;
   }else{
      charblt(0xdb, fbterm.cursor_x, fbterm.cursor_y);
      fbterm.cursor_visible = 1;
   }   
}

/* for 32bits fb */

int charblt(int c, int col, int row)
{
   int i,j;
   volatile unsigned int * offs; /* d-cache bypass */
   volatile unsigned int * offe; /* d-cache bypass */

   offs=mmapfb.base;
   offs+=(mmapfb.width*((++row<<3)-1))+(col<<3)-1;

   i=8;

	while(i--)
	{
      offe=offs+8; j=0;
      while(offe - offs)
		 {
         if ( (*(fontdata_8x8+((c<<3)+i)) >> j++) & 0x01 ) {
            *offe--=(unsigned int)0x80ffffff;
         }else{
            *offe--=(unsigned int)0x80000000;
         }
      }
      offs-=mmapfb.width;
	}

/*
   offs=mmapfb.base;
   offs+=(mmapfb.width*(row<<3))+(col<<3);

	for(i=0; i<8; i++)
	{
      offe=offs+8; j=0;
      while(offe - offs)
		{
         if ( (*(fontdata_8x8+((c<<3)+i)) >> ++j) & 0x01 ) {
            *offe--=(unsigned int)0xffffffff;
         }else{
            *offe--=(unsigned int)0xff000000;
         }
      }
      offs+=mmapfb.width;
	}
*/
  
  return 0;
}

static void fbhscroll(void)
{
   unsigned int *ps, *pe;
   unsigned long count;

   ps = mmapfb.base;
   pe = ps + (mmapfb.width << 3); /* mmapfb.width * 8 */
   count = (mmapfb.width * (mmapfb.height - font_h));

   while(count--) *ps++ = *pe++;

   /* blank last line */

   count = (mmapfb.width << 3); /* mmapfb.width * 8 */
   
   while(count--) *ps++ = 0x80000000;

   /* flush_dcache(); */
}

/* compatible with channel_handler_t */
int chnl_fbputchar(int reserved, int c)
{
	return fbputchar(c);
}

inline int fbputchar(int c)
{
   /* special char / TODO replace by real term emulation */

   fbterm.cursor_locked = 1;
   
   switch(c){
      case '\r':
         /* "/r/n" sequence erase the first char on the line */
         /* TODO: use hw cursor */
         charblt(0x00, fbterm.cursor_x, fbterm.cursor_y);   /* cursor */
         fbterm.cursor_x = 0;
         break;
      case '\n':
         charblt(0x00, fbterm.cursor_x, fbterm.cursor_y);   /* cursor */
         fbterm.cursor_x = 0;
         fbterm.cursor_y++;
         break;
      default:
         charblt(c, fbterm.cursor_x++, fbterm.cursor_y);
         break;
   }

   /* check for borders */

   if (fbterm.cursor_x == fbterm.x_max_chr) {
      fbterm.cursor_x = 0;
      fbterm.cursor_y++;      
   }

   if (fbterm.cursor_y == fbterm.y_max_chr) {
      fbhscroll();
      fbterm.cursor_y--;
   }

   charblt(0xdb, fbterm.cursor_x, fbterm.cursor_y);   /* cursor */

   fbterm.cursor_locked = 0;

   return (int)c;
}

int fbputs(char* str)
{
	int i=0;

	while(str[i])
	{
      fbputchar(str[i]);
      i++;
	}
   
	return 0;
}

int fb_init(unsigned int* fb, int width, int height)
{
   /* TODO check if width/height are ok */

   mmapfb.width = width;
   mmapfb.height = height;
   mmapfb.base = fb;

   fbterm.x_max_chr = width / font_w;
   fbterm.y_max_chr = height / font_h;

   fbterm.cursor_x = 0;
   fbterm.cursor_y = 0;
   fbterm.cursor_visible = 1;
   fbterm.cursor_locked = 0;

   /* cursor blink */
   /* ... need interrupt */

   return 0;
}
