/*
 * Copyright (c) 2008, Noel Lemouel
 * All rights reserved. See LICENCE file for details.
 */

#ifndef __E300_H_
#define __E300_H_

/* user */

#define e300_rd(reg)\
   ({ unsigned int val;\
      __asm__ __volatile__ ( "mr %0, " #reg \
                             : "=r" (val) ); \
    val;})

#define e300_wr(reg, val)\
   ({ __asm__ __volatile__ ( "mr " #reg ", %0" \
                             : /* no output */ \
                             : "r" (val) \
                           ); \
   })

#define e300_iwr(reg, val)\
   ({ __asm__ __volatile__ ( "lis " #reg ", (" #val ")@h\n\t" \
                             "ori " #reg "," #reg ", (" #val ")@l\n\t" \
                           ); \
   })

/* supervisor */

#define e300_msr_rd()\
   ({ unsigned int val;\
      __asm__ __volatile__ ( "mfmsr %0" \
                             : "=r" (val) ); \
    val;})

#define e300_msr_wr(val)\
   ({ __asm__ __volatile__ ( "mtmsr %0" \
                             : /* no output */ \
                             : "r" (val) \
                           ); \
   })

#define e300_spr_rd(spr)\
   ({ unsigned int val;\
      __asm__ __volatile__ ( "mfspr %0, %1" \
                             : "=r" (val) \
                             : "i" (spr) ); \
    val;})

#define e300_spr_wr(spr, val)\
   ({ __asm__ __volatile__ ( "mtspr %0, %1" \
                             : /* no output */ \
                             : "i" (spr), "r" (val) \
                           ); \
   })

#define e300_oea_rd(spr)\
   ({ unsigned int val;\
      __asm__ __volatile__ ( "mfspr %0, %1" \
                             : "=r" (val) \
                             : "i" (spr) ); \
    val;})

/* utility */

#define sym_rd(sym)\
   ({ unsigned int val;\
   __asm__ __volatile__ ( "stwu r20,-4(sp)\n\t" \
                          "lis r20, (" #sym ")@ha\n\t" \
                          "addi r20, r20, (" #sym ")@l\n\r" \
                          "mr %0, r20\n\t" \
                          "lwzu r20,4(sp)" \
                          : "=r" (val) \
                        ); \
    val;})

#define v_endian_swp32(val)\
   ({ unsigned int swp;\
   __asm__ __volatile__ ( "addi sp, sp, -4\n\t" \
                          "lis %0, (" #val ")@ha\n\t" \
                          "addi %0, %0, (" #val ")@l\n\t" \
                          "stw %0, 0(sp)\n\t" \
                          "lwbrx %0, r0, sp\n\t" \
                          "addi sp, sp, 4" \
                          : "=r" (swp) \
                        ); \
    swp;})

static inline unsigned int p_endian_swp32(unsigned int *p)
{
   unsigned int val;
   __asm__ __volatile__ ( "lwbrx %0, r0, %1\n\t" \
                          : "=r" (val) \
                          : "r" (p) \
                        );
   return val;
}

/* e300 core OEA registers definition */

#define MSR		msr

/* SPR definition */

#define XER		1
#define LR		8
#define CTR		9
#define DSISR	16
#define DAR		19
#define DEC		22
#define SDR1	25
#define SRR0	26
#define SRR1	27
#define CSRR0	58
#define CSRR1	59
#define SPRG0	272
#define SPRG1	273
#define SPRG2	274
#define SPRG3	275
#define SPRG4	276
#define SPRG5	277
#define SPRG6	278
#define SPRG7	279
#define TBL		284
#define TBU		285
#define SVR		286
#define PVR		287
#define IBCR	309
#define DBCR	310
#define MBAR	311
#define DABR2	317
#define IBAT0U	528
#define IBAT0L	529
#define IBAT1U	530
#define IBAT1L	531
#define IBAT2U	532
#define IBAT2L	533
#define IBAT3U	534
#define IBAT3L	535
#define DBAT0U	536
#define DBAT0L	537
#define DBAT1U	538
#define DBAT1L	539
#define DBAT2U	540
#define DBAT2L	541
#define DBAT3U	542
#define DBAT3L	543
#define IBAT4U	560
#define IBAT4L	561
#define IBAT5U	562
#define IBAT5L	563
#define IBAT6U	564
#define IBAT6L	565
#define IBAT7U	566
#define IBAT7L	567
#define DBAT4U	568
#define DBAT4L	569
#define DBAT5U	570
#define DBAT5L	571
#define DBAT6U	572
#define DBAT6L	573
#define DBAT7U	574
#define DBAT7L	575
#define DMISS	976
#define DCMP	977
#define HASH1	978
#define HASH2	979
#define IMISS	980
#define ICMP	981
#define RPA		982
#define HID0	1008
#define HID1	1009
#define IABR	1010
#define HID2	1011
#define DABR	1013
#define IABR2	1018

/* MSR bit settings */

#define MSR_LE		0x00000001
#define MSR_RI		0x00000002
#define MSR_PMM		0x00000004
#define MSR_DR		0x00000010
#define MSR_IR		0x00000020
#define MSR_IP		0x00000040
#define MSR_CE		0x00000080
#define MSR_FE1		0x00000100
#define MSR_BE		0x00000200
#define MSR_SE		0x00000400
#define MSR_FE0		0x00000800
#define MSR_ME		0x00001000
#define MSR_FP		0x00002000
#define MSR_PR		0x00004000
#define MSR_EE		0x00008000
#define MSR_ILE		0x00010000
#define MSR_TGPR	0x00020000
#define MSR_POW		0x00040000

#endif /* __E300_H_ */
