/*
 * Copyright (c) 2008, Noel Lemouel
 * All rights reserved. See LICENCE file for details.
 */

#ifndef __MMAP_H__
#define __MMAP_H__

/************************************************************/

/* cpu */

/* change with e300 core values */

#define CPU_ICACHE_SIZE       2048        /* 2 KByte total	*/
#define CPU_ICACHELINE_SIZE   32          /* 32 bytes/line	*/

#define CPU_DCACHE_SIZE       8192        /* 8 KByte (II/f) */
#define CPU_DCACHELINE_SIZE   32          /* 32 bytes/line (II/f) */

#define SYS_EXCEPTION_ADDR    0x00000020  /* Exception entry point */

#define SYS_INIT_SP           0x30010000  /* 64kB in SRAM */

#define SYS_CLK_FREQ          400000000   /* 400 MHz */ 
#define SYS_IP_BUS_CLK_FREQ   066000000   /* 66 Mhz */

/************************************************************/

/* memories */

#define MEM_IMMR              0x80000000  /* get it through asm */

#define MEM_NOR_FLASH_BASE    0xfc000000  /* FLASH base addr */
#define MEM_NOR_FLASH_SIZE    0x03ffffff  /* 64 MByte */
#define MEM_DRAM_BASE         0x00000000  /* DRAM base addr */
#define MEM_DRAM_SIZE         0x0fffffff  /* 256 MByte	*/
#define MEM_SRAM_BASE         0x30000000  /* SRAM base addr */
#define MEM_SRAM_SIZE         0x0001ffff  /* 128 kBytes */

/************************************************************/

/* peripherals */

#define IO_IPIC_OFFSET        0x00000c00
#define IO_CLOCK_OFFSET  	  0x00000f00
#define IO_DIU_OFFSET         0x00002100

#define IO_GPT0_OFFSET        0x00000b00
#define IO_GPT1_OFFSET        0x00000b10

#define IO_PSC3_OFFSET        0x00011300

#define IO_CPLD_OFFSET        0x02000000  /* ads5121 */

/* graphics */

#define IO_ADSC_1_BASE        0x01ffff00
#define IO_ADSC_2_BASE        0x01ffff40
#define IO_ADSC_3_BASE        0x01ffff80

#define MEM_GFX_P1_BASE       0x02000000  /* 1024*768*32bpp */
#define MEM_GFX_P2_BASE       0x02300000  /* 1024*768*32bpp */
#define MEM_GFX_P3_BASE       0x02600000  /* 1024*768*32bpp */

/************************************************************/

/* console */

#define IO_DFLT_UART          IO_PSC3_OFFSET

#endif   /* __MMAP_H__ */
 
